% Interactive interface for 2D sparse array design
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
%
% Our Papers:
% [1] C.-L. Liu and P. P. Vaidyanathan, “Two-Dimensional Sparse Arrays with Hole-Free Coarray and Reduced Mutual Coupling,” in Proc. of 50th IEEE Asilomar Conference on Signals, Systems, and Computers, Pacific Grove, CA, Nov. 2016.
% [2] C.-L. Liu and P. P. Vaidyanathan, “Hourglass Arrays, and Other Novel 2D Sparse Arrays with Reduced Mutual Coupling,” submitted to IEEE Transactions on Signal Processing.
%
% Last revised on August 21, 2016
%
% Usage:	Run this program directly
%			Check (V) on the 2D plane as sensors
%
% Push Buttons:
% 1. New Panel: Open a new plane with given Nx and Ny
% 2. Show Array Profile: Display the following:
%       1) Physical sensors
%       2) Difference coarray
%       2) Weight function w(m_x, m_y)
% 3. Load Arrays: Load various array configurations
% 4. Clear Panel: Remove all sensors




function interactive_interface( Nx, Ny )


if ( nargin == 0 )
    % Default Nx and Ny
    clear; clc; close all;
    Nx = 11; Ny = 11;
end



button_width = 200;
button_height = 25;
width = 12;
margin = 8;

% Identify size
Xmax = max( width * Nx, button_width*2 + margin ) + 2 * margin;
Ymax = margin * 4 + button_height*2 + width * Ny;

f = figure('Position', [300, 300, Xmax, Ymax]);
set(f, 'Name', ['N_x = ', num2str(Nx), ', N_y = ', num2str(Ny)])

% Sensor locations
sensor_chkbox = zeros(Ny, Nx);
for nn2 = 1 : Ny
    for nn1 = 1 : Nx
        sensor_chkbox(nn2, nn1) = uicontrol( f, ...
            'Style', 'checkbox',...
            'Value', (nn2 == 1 || nn1 == Nx), ...
            'Position', [width*(nn1-1)+margin, width*(nn2-1)+button_height*2+margin*3, width, width]);
    end
end

% Display sensor locations
create_new_btn = uicontrol( f, ...
    'Style', 'pushbutton', ...
    'String', 'New Panel', ...
    'Position', [margin, margin*2+button_height, button_width, button_height], ...
    'Callback', @create_new_btn_Callback);
    function create_new_btn_Callback(hObject, eventdata, handles)
        % Prompt user to enter Nx and Ny
        while(1)
            dialog_answer = inputdlg({'Enter Nx:','Enter Ny:'}, ...
                'Please specify Nx and Ny', ...
                1, ...
                {'11', '11'});
            if (length(dialog_answer) == 2)
                Nx_entered = str2double( dialog_answer{1} );
                Ny_entered = str2double( dialog_answer{2} );
                
                % Check Nx and Ny. They need to be positive integers
                if ( Nx_entered == floor(Nx_entered) && Nx_entered >= 1 && Ny_entered == floor(Ny_entered) && Ny_entered >= 1)
                    break;
                else
                    uiwait( msgbox('N_x and N_y have to be positive integers.', 'Error','error') )
                end
            else
                return;
            end
        end
        
        interactive_interface( Nx_entered, Ny_entered );
    end

% display physical arrays, coarrays, and weight functions
show_coarray_btn = uicontrol( f, ...
    'Style', 'pushbutton', ...
    'String', 'Show Array Profile', ...
    'Position', [margin, margin, button_width, button_height], ...
    'Callback', @show_coarray_btn_Callback);
    function show_coarray_btn_Callback(hObject, eventdata, handles)
        % Get sensor locations
        S = [];
        for ny = 1 : Ny
            for nx = 1 : Nx
                if ( get( sensor_chkbox(ny,nx), 'Value') )
                    S = [S; nx, ny];
                end
            end
        end
        S = S - 1;
        
        if ( isempty( S ) )
            uiwait( msgbox('No sensors', 'Error','error') )
            return;
        end
        
        LEN_S = size(S, 1);
        disp(['Number of sensors: ', num2str( LEN_S )])
        disp('Sensor location (n_x, n_y):');
        disp(num2str(S));
        
        % Figure;
        ff = figure;
        set(ff, 'Name', ['Array profile: Nx = ', num2str(Nx), ', Ny = ', num2str(Ny)])
        % Physical arrays
        subplot(2, 2, 1)
        plot(S(:, 1), S(:, 2), '.');
        grid on;
        axis([-1, Nx, -1, Ny])
        xlabel('$n_x$', 'interpreter', 'latex')
        ylabel('$n_y$', 'interpreter', 'latex')
        title(['Physical array: \# of sensors = ', num2str( LEN_S )], 'interpreter', 'latex')
        axis equal
        
        % Difference coarray
        subplot(2, 2, 2)
        [ V, w, n1, n2 ] = weight_function_2D( S, 'V' );
        plot(V(w ~= 0, 1), V(w ~= 0, 2), 'o')
        grid on;
        axis([-Nx, Nx, -Ny, Ny])
        xlabel('$m_x$', 'interpreter', 'latex')
        ylabel('$m_y$', 'interpreter', 'latex')
        % Determine whether the difference coarray is hole-free or not.
        if (sum(w == 0) == 0)
            % Hole-free
            type = 'with no holes';
        else
            % Not hole-free
            type = 'with holes';
        end
        title(['Difference coarray, ', type], 'interpreter', 'latex')
        axis equal
        
        % Weight functions
        subplot(2, 2, 3)
        m_x = V(w ~= 0, 1);
        m_y = V(w ~= 0, 2);
        stem3(m_x, m_y, w( w ~= 0))
        xlabel('$m_x$', 'interpreter', 'latex')
        ylabel('$m_y$', 'interpreter', 'latex')
        zlabel('$w(m_x, m_y)$', 'interpreter', 'latex')
        title({'$w(m_x, m_y)$', 'click on the numbers on the right'}, 'interpreter', 'latex')
        
        % Visualize w(m) around the origin
        Mx = 2; % max(abs(V(:, 1)));
        My = 2; % max(abs(V(:, 2)));
        mx_grid = -Mx : Mx;
        my_grid = -My : My;
        [m_x, m_y] = meshgrid(mx_grid, my_grid);
        w_zoom_in = zeros(2*My+1, 2*Mx+1);
        
        for ii = 1 : length(mx_grid)
            for jj = 1 : length(my_grid)
                weight = w( V(:, 1) == mx_grid(ii) & V(:, 2) == my_grid(jj) );
                if (weight == 0)
                    w_zoom_in( m_x == mx_grid(ii) & m_y == my_grid(jj) ) = 0; % Inf;
                else
                    w_zoom_in( m_x == mx_grid(ii) & m_y == my_grid(jj) ) = weight;
                end
            end
        end
        
        pos = get(subplot(2, 2, 4),'position');
        
        t = uitable(ff,'Data',flipud(w_zoom_in),...
            'ColumnName', [repmat('m_x = ', [2*Mx+1, 1]), num2str(mx_grid.')],...
            'RowName', [repmat('m_y = ', [2*My+1, 1]), num2str(fliplr(my_grid).')], ...
            'ColumnWidth',{60}, ...
            'CellSelectionCallback', @uitable1_CellSelectionCallback);
        delete(subplot(2, 2, 4))
        set(t,'units','normalized')
        set(t,'position',pos)
        
        function uitable1_CellSelectionCallback(hObject, eventdata, handles)
            indices = eventdata.Indices;
            r = indices(:,1);
            c = indices(:,2);
            mx = -Mx + c - 1;
            my = My - r + 1;
            
            
            data = [n1{ V(:, 1) == mx & V(:, 2) == my }, ...
                n2{ V(:, 1) == mx & V(:, 2) == my }];
            
            
            
            str = sprintf('\n w(%6d, %6d) = %6d \n', mx, my, size(data, 1));
            str = [str, sprintf('All the sensor pairs (n_1, n_2) such that n_1 - n_2 = (%3d, %3d)\n', mx, my)];
            str = [str, sprintf('      n_1       ||        n_2\n=================================\n')];
            for index = 1 : size(data, 1)
                str = [str, sprintf('(%6d, %5d) || (%6d, %5d)', data(index, 1), data(index, 2), data(index, 3), data(index, 4))];
                str = [str, sprintf('\n')];
            end
            
            msgbox(str, ['Weight function w(', num2str(mx), ', ', num2str(my), ')'])
            
            disp(str)
            
        end
        
        
        
    end

% Load some known configurations
load_super_nested_btn = uicontrol( f, ...
    'Style', 'pushbutton', ...
    'String', 'Load Arrays', ...
    'Position', [margin*2+button_width, margin*2+button_height, button_width, button_height], ...
    'Callback', @load_super_nested_btn_Callback);
    function load_super_nested_btn_Callback(hObject, eventdata, handles)
        while (1)
            
            TYPES = 9;
            
            dialog_answer = inputdlg(['Please enter array configurations(1-', num2str(TYPES), '): ', sprintf('\n'), ...
                ' 1: Uniform rectangular array, ', sprintf('\n'), ...
                ' 2: Billboard array, ', sprintf('\n'), ...
                ' 3: Open box array, ', sprintf('\n'), ...
                ' 4: 2D nested arrays, ', sprintf('\n'), ...
                ' 5: Partially open box arrays (specify g_1), ', sprintf('\n'), ...
                ' 6: Half open box arrays, ', sprintf('\n'), ...
                ' 7: Partially open box arrays with L layers (specify g_1 and h_1), ', sprintf('\n'), ...
                ' 8: Half open box arrays with two layers, ', sprintf('\n'), ...
                ' 9: Hourglass arrays. ', sprintf('\n')
                ], 'Array Selection', 1, {'9'});
            
            if (length(dialog_answer) == 1)
                array_type = str2double(dialog_answer{1});
                if ( array_type == floor(array_type) && array_type >= 1 && array_type <= TYPES )
                    break;
                else
                    uiwait( msgbox('Not one of these array configurations', 'Error','error') )
                end
            else
                return;
            end
        end
        switch ( array_type )
            case 1
                S = URA( Nx, Ny );
            case 2
                S = billboard( Nx, Ny );
            case 3
                S = POBA_L( Nx, Ny, 'OBA' );
            case 4
                N1x = floor( (sqrt(4*Nx+1)-1)/2 );
                N2x = Nx - N1x;
                N1y = floor( (sqrt(4*Ny+1)-1)/2 );
                N2y = Ny - N1y;
                Sx = super_nested( N1x, N2x, 1);
                Sy = super_nested( N1y, N2y, 1);
                [Sx_all, Sy_all] = ndgrid(Sx, Sy);
                S = [Sx_all(:), Sy_all(:)] - 1;
            case 5
                % Enter g1
                g_1 = enter_g_1( Nx );
                if ( isnan(g_1) ) % cancel
                    return;
                end
                S = POBA_L( Nx, Ny, g_1 );
            case 6
                S = POBA_L( Nx, Ny, 'HOBA' );
            case 7
                % Enter g1
                g_1 = enter_g_1( Nx );
                if ( isnan(g_1) ) % cancel
                    return;
                end
                % Enter layers L
                L = enter_L( Nx );
                if ( isnan(L) ) % cancel
                    return;
                end
                % Enter h_1
                h_1 = enter_h_1( Ny, L );
                for ll = 1 : L
                    if ( isnan(h_1{ll}) ) % cancel
                        return;
                    end
                end
                S = POBA_L( Nx, Ny, g_1, h_1 );
            case 8
                S = POBA_L( Nx, Ny, 'HOBA-2' );
            case 9
                S = POBA_L( Nx, Ny, 'hourglass' );
        end
        
        % Show on the interface
        for jj = 1 : Ny
            for ii = 1 : Nx
                set( sensor_chkbox(jj,ii), 'Value', sum(S(:, 1) == ii-1 & S(:, 2) == jj-1));
            end
        end
    end

clear_panel_btn = uicontrol( f, ...
    'Style', 'pushbutton', ...
    'String', 'Clear Panel', ...
    'Position', [margin*2+button_width, margin, button_width, button_height], ...
    'Callback', @clear_panel_btn_Callback);
    function clear_panel_btn_Callback(hObject, eventdata, handles)
        for jj = 1 : Ny
            for ii = 1 : Nx
                set( sensor_chkbox(jj,ii), 'Value', 0);
            end
        end
    end

end


function g_1 = enter_g_1( Nx )
while(1)
    g_1_str = inputdlg(['Please enter the vector g_1.', sprintf('\n'), ...
        'g_1 has to be a subset of {1, ..., N_x-2} = {1, ..., ', num2str(Nx-2), '}', sprintf('\n\n'), ...
        'If g_1 = {1, 3, 5}, please enter: 1,3,5', sprintf('\n'), ...
        'If g_1 = {} = the empty set, please enter: EMPTY', sprintf('\n'), ...
        'If g_1 = {1, ..., N_x-2}, please enter: ALL', sprintf('\n')], ...
        'Enter g_1', ...
        1, ...
        {'1,3,5'});
    if ( isempty(g_1_str) )     % Cancel
        g_1 = NaN;
        return;
    end
    
    g_1 = str2double( strsplit(g_1_str{1},',') );
    if ( strcmpi(g_1_str{1}, 'empty') )
        g_1 = [];
        return;
    elseif ( strcmpi(g_1_str{1}, 'all') )
        g_1 = 1 : Nx-2;
        return;
    elseif ( isempty( setdiff(g_1, 1:Nx-2) ) )
        return;
    else
        uiwait( msgbox('The set g_1 needs to be a subset of {1, ..., N_y-2}', 'Error','error') )
    end
end
end

function L = enter_L ( Nx )
while(1)
    if (Nx/2 >= 2)
        default_text = '2';
    else
        default_text = '1';
    end
    L_str = inputdlg(['Please enter the number of layers L.', sprintf('\n'), ...
        'L must be an integer satisfying 1 <= L <= N_x/2 = ', num2str(Nx/2), sprintf('\n')], ...
        'Enter L', ...
        1, ...
        {default_text});
    if ( isempty(L_str) )     % Cancel
        L = NaN;
        return;
    end
    L = str2double(L_str);
    if (L == floor(L) && L >= 1 && L <= Nx/2)
        return;
    else
        uiwait( msgbox(['The integer L must satisfy 1 <= L <= N_x/2 = ', num2str(Nx/2)], 'Error','error') )
    end
end
end


function h_1 = enter_h_1 ( Ny, L )
prompt = cell(1, L);
for ll = 1 : L
    prompt{ll} = ['h_{1, ', num2str(ll) '} = '];
end
prompt{1} = ['Please enter h_{1, \ell} in the following boxes:', sprintf('\n'), ...
    '{ h_{1, \ell} } has to be a partition of {1, ..., N_y-2} = {1, ..., ', num2str(Ny-2), '}', sprintf('\n\n'), ...
    'If h_{1, \ell} = {1, 3, 5}, please enter 1,3,5', sprintf('\n'), ...
    'If h_{1, \ell} = {} = the empty set, please enter EMPTY', sprintf('\n'), ...
    'If h_{1, \ell} = {1, ..., N_y-2}, please enter: ALL', sprintf('\n\n'), ...
    prompt{1}];
while(1)
    h_1_str = inputdlg(prompt);
    if ( isempty(h_1_str) )     % Cancel
        h_1 = NaN;
        return;
    end
    
    h_1 = cell(1, L);
    h_1_message = [];
    for ll = 1 : L
        if ( strcmpi(h_1_str{ll}, 'empty') )
            h_1{ll} = [];
            h_1_message = [h_1_message, sprintf('\n'), ...
                'h_{1, ', num2str(ll), '} = {}'];
        elseif ( strcmpi(h_1_str{ll}, 'all') )
            h_1{ll} = 1 : Ny-2;
            h_1_message = [h_1_message, sprintf('\n'), ...
                'h_{1, ', num2str(ll), '} = {1, ..., ', num2str(Ny-2), '}'];
        else
            h_1{ll} = str2double( strsplit(h_1_str{ll},',') );
            h_1_message = [h_1_message, sprintf('\n'), ...
                'h_{1, ', num2str(ll), '} = {', h_1_str{ll}, '}'];
        end
    end
    
    % h_{1,ll} needs to be a partition of {1, ..., Ny-2}
    H = []; % Union of h_{1,\ell}
    VALID = true;
    FIRST_ERROR = false;
    for ll = 1 : L
        % H and h{ll} have to be disjoint
        if ( ~isempty( intersect(H, h_1{ll}) ) )
            if (~FIRST_ERROR)
                uiwait( msgbox(['You have entered:', h_1_message, sprintf('\n\n'), 'h_{1, ll} are not disjoint.'], 'Error','error') )
                VALID = false;
                FIRST_ERROR = true;
            end
        end
        H = [H, h_1{ll}];
    end
    if (~isequal(unique(H), (1:Ny-2)))
        uiwait( msgbox(['You have entered:', h_1_message, sprintf('\n\n'), 'The union of h_{1, ll} is not {1, ..., N_y-2} = {1, ..., ', num2str(Ny-2), '}'], 'Error','error') )
        VALID = false;
    end
    if (VALID)
        break;
    end
end
end
